@@ -18,11 +18,13 @@ from __future__ import absolute_import
 import os
 import re
 import sys
+import time
 import types
 import socket
 import logging
 import logging.handlers
 import traceback
+import multiprocessing
 
 # Import 3rd-party libs
 import salt.ext.six as six
@@ -37,7 +39,12 @@ QUIET = logging.QUIET = 1000
 
 # Import salt libs
 from salt.textformat import TextFormat
-from salt.log.handlers import TemporaryLoggingHandler, StreamHandler, SysLogHandler, WatchedFileHandler
+from salt.log.handlers import (TemporaryLoggingHandler,
+                               StreamHandler,
+                               SysLogHandler,
+                               FileHandler,
+                               WatchedFileHandler,
+                               QueueHandler)
 from salt.log.mixins import LoggingMixInMeta, NewStyleClassMixIn
 
 
@@ -67,6 +74,8 @@ LOG_COLORS = {
         'TRACE': TextFormat('bold', 'magenta'),
         'GARBAGE': TextFormat('bold', 'blue'),
         'NOTSET': TextFormat('reset'),
+        'SUBDEBUG': TextFormat('bold', 'cyan'),  # used by multiprocessing.log_to_stderr()
+        'SUBWARNING': TextFormat('bold', 'yellow'),  # used by multiprocessing.log_to_stderr()
     },
     'msgs': {
         'QUIET': TextFormat('reset'),
@@ -79,6 +88,8 @@ LOG_COLORS = {
         'TRACE': TextFormat('magenta'),
         'GARBAGE': TextFormat('blue'),
         'NOTSET': TextFormat('reset'),
+        'SUBDEBUG': TextFormat('bold', 'cyan'),  # used by multiprocessing.log_to_stderr()
+        'SUBWARNING': TextFormat('bold', 'yellow'),  # used by multiprocessing.log_to_stderr()
     },
     'name': TextFormat('bold', 'green'),
     'process': TextFormat('bold', 'blue'),
@@ -99,6 +110,12 @@ __CONSOLE_CONFIGURED = False
 __LOGFILE_CONFIGURED = False
 __TEMP_LOGGING_CONFIGURED = False
 __EXTERNAL_LOGGERS_CONFIGURED = False
+__MP_LOGGING_LISTENER_CONFIGURED = False
+__MP_LOGGING_CONFIGURED = False
+__MP_LOGGING_QUEUE = None
+__MP_LOGGING_QUEUE_PROCESS = None
+__MP_LOGGING_QUEUE_HANDLER = None
+__MP_IN_MAINPROCESS = multiprocessing.current_process().name == 'MainProcess'
 
 
 def is_console_configured():
@@ -117,6 +134,14 @@ def is_temp_logging_configured():
     return __TEMP_LOGGING_CONFIGURED
 
 
+def is_mp_logging_listener_configured():
+    return __MP_LOGGING_LISTENER_CONFIGURED
+
+
+def is_mp_logging_configured():
+    return __MP_LOGGING_LISTENER_CONFIGURED
+
+
 def is_extended_logging_configured():
     return __EXTERNAL_LOGGERS_CONFIGURED
 
@@ -131,6 +156,10 @@ LOGGING_TEMP_HANDLER = StreamHandler(sys.stderr)
 LOGGING_STORE_HANDLER = TemporaryLoggingHandler()
 
 
+class SaltLogQueueHandler(QueueHandler):
+    pass
+
+
 class SaltLogRecord(logging.LogRecord):
     def __init__(self, *args, **kwargs):
         logging.LogRecord.__init__(self, *args, **kwargs)
@@ -155,7 +184,7 @@ class SaltColorLogRecord(SaltLogRecord):
                                           reset)
         self.colorlevel = '%s[%-8s]%s' % (clevel,
                                           self.levelname,
-                                          TextFormat('reset'))
+                                          reset)
         self.colorprocess = '%s[%5s]%s' % (LOG_COLORS['process'],
                                            self.process,
                                            reset)
@@ -163,7 +192,7 @@ class SaltColorLogRecord(SaltLogRecord):
         # pylint: enable=E1321
 
 
-_log_record_factory = SaltLogRecord
+_LOG_RECORD_FACTORY = SaltLogRecord
 
 
 def set_log_record_factory(factory):
@@ -173,8 +202,8 @@ def set_log_record_factory(factory):
     :param factory: A callable which will be called to instantiate
     a log record.
     '''
-    global _log_record_factory
-    _log_record_factory = factory
+    global _LOG_RECORD_FACTORY
+    _LOG_RECORD_FACTORY = factory
 
 
 def get_log_record_factory():
@@ -182,7 +211,7 @@ def get_log_record_factory():
     Return the factory to be used when instantiating a log record.
     '''
 
-    return _log_record_factory
+    return _LOG_RECORD_FACTORY
 
 
 set_log_record_factory(SaltLogRecord)
@@ -256,18 +285,21 @@ class SaltLoggingClass(six.with_metaclass(LoggingMixInMeta, LOGGING_LOGGER_CLASS
 
     def _log(self, level, msg, args, exc_info=None, extra=None,  # pylint: disable=arguments-differ
              exc_info_on_loglevel=None):
-        # If both exc_info and exc_info_on_loglevel are both passed, let's fail.
+        # If both exc_info and exc_info_on_loglevel are both passed, let's fail
         if exc_info and exc_info_on_loglevel:
             raise RuntimeError(
-                'Please only use one of \'exc_info\' and \'exc_info_on_loglevel\', not both'
+                'Only one of \'exc_info\' and \'exc_info_on_loglevel\' is '
+                'permitted'
             )
         if exc_info_on_loglevel is not None:
             if isinstance(exc_info_on_loglevel, six.string_types):
-                exc_info_on_loglevel = LOG_LEVELS.get(exc_info_on_loglevel, logging.ERROR)
+                exc_info_on_loglevel = LOG_LEVELS.get(exc_info_on_loglevel,
+                                                      logging.ERROR)
             elif not isinstance(exc_info_on_loglevel, int):
                 raise RuntimeError(
-                    'The value of \'exc_info_on_loglevel\' needs to be a logging level or '
-                    'a logging level name, not {0!r}'.format(exc_info_on_loglevel)
+                    'The value of \'exc_info_on_loglevel\' needs to be a '
+                    'logging level or a logging level name, not \'{0}\''
+                    .format(exc_info_on_loglevel)
                 )
         if extra is None:
             extra = {'exc_info_on_loglevel': exc_info_on_loglevel}
@@ -289,27 +321,33 @@ class SaltLoggingClass(six.with_metaclass(LoggingMixInMeta, LOGGING_LOGGER_CLASS
             extra = None
 
         # Let's try to make every logging message unicode
-        if isinstance(msg, six.string_types) and not isinstance(msg, six.text_type):
+        if isinstance(msg, six.string_types) \
+                and not isinstance(msg, six.text_type):
+            salt_system_encoding = __salt_system_encoding__
+            if salt_system_encoding == 'ascii':
+                # Encoding detection most likely failed, let's use the utf-8
+                # value which we defaulted before __salt_system_encoding__ was
+                # implemented
+                salt_system_encoding = 'utf-8'
             try:
-                _msg = msg.decode('utf-8', 'replace')
+                _msg = msg.decode(salt_system_encoding, 'replace')
             except UnicodeDecodeError:
-                _msg = msg.decode('utf-8', 'ignore'),
+                _msg = msg.decode(salt_system_encoding, 'ignore')
         else:
             _msg = msg
 
         if six.PY3:
-            logrecord = _log_record_factory(name, level, fn, lno, _msg, args,
-                                          exc_info, func, sinfo)
+            logrecord = _LOG_RECORD_FACTORY(name, level, fn, lno, _msg, args,
+                                            exc_info, func, sinfo)
         else:
-            logrecord = _log_record_factory(name, level, fn, lno, _msg, args,
-                                          exc_info, func)
+            logrecord = _LOG_RECORD_FACTORY(name, level, fn, lno, _msg, args,
+                                            exc_info, func)
 
         if extra is not None:
             for key in extra:
-                if (key in ['message', 'asctime']) \
-                        or (key in logrecord.__dict__):
+                if (key in ['message', 'asctime']) or (key in logrecord.__dict__):
                     raise KeyError(
-                        'Attempt to overwrite {0!r} in LogRecord'.format(key)
+                        'Attempt to overwrite \'{0}\' in LogRecord'.format(key)
                     )
                 logrecord.__dict__[key] = extra[key]
 
@@ -541,7 +579,7 @@ def setup_logfile_logger(log_path, log_level='error', log_format=None,
                 # Logging facilities start with LOG_ if this is not the case
                 # fail right now!
                 raise RuntimeError(
-                    'The syslog facility {0!r} is not known'.format(
+                    'The syslog facility \'{0}\' is not known'.format(
                         facility_name
                     )
                 )
@@ -556,7 +594,7 @@ def setup_logfile_logger(log_path, log_level='error', log_format=None,
             # This python syslog version does not know about the user provided
             # facility name
             raise RuntimeError(
-                'The syslog facility {0!r} is not known'.format(
+                'The syslog facility \'{0}\' is not known'.format(
                     facility_name
                 )
             )
@@ -590,8 +628,22 @@ def setup_logfile_logger(log_path, log_level='error', log_format=None,
                     err
                 )
             )
+            shutdown_multiprocessing_logging_listener()
             sys.exit(2)
     else:
+        # make sure, the logging directory exists and attempt to create it if necessary
+        log_dir = os.path.dirname(log_path)
+        if not os.path.exists(log_dir):
+            logging.getLogger(__name__).info(
+                'Log directory not found, trying to create it: {0}'.format(log_dir)
+            )
+            try:
+                os.makedirs(log_dir, mode=0o700)
+            except OSError as ose:
+                logging.getLogger(__name__).warning(
+                    'Failed to create directory for log file: {0} ({1})'.format(log_dir, ose)
+                )
+                return
         try:
             # Logfile logging is UTF-8 on purpose.
             # Since salt uses YAML and YAML uses either UTF-8 or UTF-16, if a
@@ -682,7 +734,7 @@ def setup_extended_logging(opts):
                 continue
 
             logging.getLogger(__name__).debug(
-                'Adding the {0!r} provided logging handler: {1!r}'.format(
+                'Adding the \'{0}\' provided logging handler: \'{1}\''.format(
                     name, handler
                 )
             )
@@ -706,10 +758,155 @@ def setup_extended_logging(opts):
     # Remove the temporary queue logging handler
     __remove_queue_logging_handler()
 
+    # Remove the temporary null logging handler (if it exists)
+    __remove_null_logging_handler()
+
     global __EXTERNAL_LOGGERS_CONFIGURED
     __EXTERNAL_LOGGERS_CONFIGURED = True
 
 
+def get_multiprocessing_logging_queue():
+    global __MP_LOGGING_QUEUE
+
+    if __MP_IN_MAINPROCESS is False:
+        # We're not in the MainProcess, return! No Queue shall be instantiated
+        return __MP_LOGGING_QUEUE
+
+    if __MP_LOGGING_QUEUE is None:
+        __MP_LOGGING_QUEUE = multiprocessing.Queue()
+    return __MP_LOGGING_QUEUE
+
+
+def set_multiprocessing_logging_queue(queue):
+    global __MP_LOGGING_QUEUE
+    if __MP_LOGGING_QUEUE is not queue:
+        __MP_LOGGING_QUEUE = queue
+
+
+def setup_multiprocessing_logging_listener(opts, queue=None):
+    global __MP_LOGGING_QUEUE_PROCESS
+    global __MP_LOGGING_LISTENER_CONFIGURED
+
+    if __MP_IN_MAINPROCESS is False:
+        # We're not in the MainProcess, return! No logging listener setup shall happen
+        return
+
+    if __MP_LOGGING_LISTENER_CONFIGURED is True:
+        return
+
+    __MP_LOGGING_QUEUE_PROCESS = multiprocessing.Process(
+        target=__process_multiprocessing_logging_queue,
+        args=(opts, queue or get_multiprocessing_logging_queue(),)
+    )
+    __MP_LOGGING_QUEUE_PROCESS.daemon = True
+    __MP_LOGGING_QUEUE_PROCESS.start()
+    __MP_LOGGING_LISTENER_CONFIGURED = True
+
+
+def setup_multiprocessing_logging(queue=None):
+    '''
+    This code should be called from within a running multiprocessing
+    process instance.
+    '''
+    global __MP_LOGGING_CONFIGURED
+    global __MP_LOGGING_QUEUE_HANDLER
+
+    if __MP_IN_MAINPROCESS is True:
+        # We're in the MainProcess, return! No multiprocessing logging setup shall happen
+        return
+
+    try:
+        logging._acquireLock()  # pylint: disable=protected-access
+
+        if __MP_LOGGING_CONFIGURED is True:
+            return
+
+        # Let's set it to true as fast as possible
+        __MP_LOGGING_CONFIGURED = True
+
+        if __MP_LOGGING_QUEUE_HANDLER is not None:
+            return
+
+        # The temp null and temp queue logging handlers will store messages.
+        # Since noone will process them, memory usage will grow. If they
+        # exist, remove them.
+        __remove_null_logging_handler()
+        __remove_queue_logging_handler()
+
+        # Let's add a queue handler to the logging root handlers
+        __MP_LOGGING_QUEUE_HANDLER = SaltLogQueueHandler(queue or get_multiprocessing_logging_queue())
+        logging.root.addHandler(__MP_LOGGING_QUEUE_HANDLER)
+        # Set the logging root level to the lowest to get all messages
+        logging.root.setLevel(logging.GARBAGE)
+        logging.getLogger(__name__).debug(
+            'Multiprocessing queue logging configured for the process running '
+            'under PID: {0}'.format(os.getpid())
+        )
+        # The above logging call will create, in some situations, a futex wait
+        # lock condition, probably due to the multiprocessing Queue's internal
+        # lock and semaphore mechanisms.
+        # A small sleep will allow us not to hit that futex wait lock condition.
+        time.sleep(0.0001)
+    finally:
+        logging._releaseLock()  # pylint: disable=protected-access
+
+
+def shutdown_multiprocessing_logging():
+    global __MP_LOGGING_CONFIGURED
+    global __MP_LOGGING_QUEUE_HANDLER
+
+    if __MP_IN_MAINPROCESS is True:
+        # We're in the MainProcess, return! No multiprocessing logging shutdown shall happen
+        return
+
+    try:
+        logging._acquireLock()
+        if __MP_LOGGING_CONFIGURED is True:
+            # Let's remove the queue handler from the logging root handlers
+            logging.root.removeHandler(__MP_LOGGING_QUEUE_HANDLER)
+            __MP_LOGGING_QUEUE_HANDLER = None
+            __MP_LOGGING_CONFIGURED = False
+    finally:
+        logging._releaseLock()
+
+
+def shutdown_multiprocessing_logging_listener(daemonizing=False):
+    global __MP_LOGGING_QUEUE
+    global __MP_LOGGING_QUEUE_PROCESS
+    global __MP_LOGGING_LISTENER_CONFIGURED
+
+    if daemonizing is False and __MP_IN_MAINPROCESS is True:
+        # We're in the MainProcess and we're not daemonizing, return!
+        # No multiprocessing logging listener shutdown shall happen
+        return
+    if __MP_LOGGING_QUEUE_PROCESS is None:
+        return
+    if __MP_LOGGING_QUEUE_PROCESS.is_alive():
+        logging.getLogger(__name__).debug('Stopping the multiprocessing logging queue listener')
+        try:
+            # Sent None sentinel to stop the logging processing queue
+            __MP_LOGGING_QUEUE.put(None)
+            # Let's join the multiprocessing logging handle thread
+            time.sleep(0.5)
+            logging.getLogger(__name__).debug('closing multiprocessing queue')
+            __MP_LOGGING_QUEUE.close()
+            logging.getLogger(__name__).debug('joining multiprocessing queue thread')
+            __MP_LOGGING_QUEUE.join_thread()
+            __MP_LOGGING_QUEUE = None
+            __MP_LOGGING_QUEUE_PROCESS.join(1)
+            __MP_LOGGING_QUEUE = None
+        except IOError:
+            # We were unable to deliver the sentinel to the queue
+            # carry on...
+            pass
+        if __MP_LOGGING_QUEUE_PROCESS.is_alive():
+            # Process is still alive!?
+            __MP_LOGGING_QUEUE_PROCESS.terminate()
+        __MP_LOGGING_QUEUE_PROCESS = None
+        __MP_LOGGING_LISTENER_CONFIGURED = False
+        logging.getLogger(__name__).debug('Stopped the multiprocessing logging queue listener')
+
+
 def set_logger_level(logger_name, log_level='error'):
     '''
     Tweak a specific logger's logging level
@@ -719,17 +916,58 @@ def set_logger_level(logger_name, log_level='error'):
     )
 
 
+def patch_python_logging_handlers():
+    '''
+    Patch the python logging handlers with out mixed-in classes
+    '''
+    logging.StreamHandler = StreamHandler
+    logging.FileHandler = FileHandler
+    logging.handlers.SysLogHandler = SysLogHandler
+    logging.handlers.WatchedFileHandler = WatchedFileHandler
+    if sys.version_info >= (3, 2):
+        logging.handlers.QueueHandler = QueueHandler
+
+
+def __process_multiprocessing_logging_queue(opts, queue):
+    import salt.utils
+    salt.utils.appendproctitle('MultiprocessingLoggingQueue')
+    if salt.utils.is_windows():
+        # On Windows, creating a new process doesn't fork (copy the parent
+        # process image). Due to this, we need to setup extended logging
+        # inside this process.
+        setup_temp_logger()
+        setup_extended_logging(opts)
+    while True:
+        try:
+            record = queue.get()
+            if record is None:
+                # A sentinel to stop processing the queue
+                break
+            # Just log everything, filtering will happen on the main process
+            # logging handlers
+            logger = logging.getLogger(record.name)
+            logger.handle(record)
+        except (EOFError, KeyboardInterrupt, SystemExit):
+            break
+        except Exception as exc:  # pylint: disable=broad-except
+            logging.getLogger(__name__).warn(
+                'An exception occurred in the multiprocessing logging '
+                'queue thread: {0}'.format(exc),
+                exc_info_on_loglevel=logging.DEBUG
+            )
+
+
 def __remove_null_logging_handler():
     '''
     This function will run once the temporary logging has been configured. It
     just removes the NullHandler from the logging handlers.
     '''
-    if is_temp_logging_configured():
-        # In this case, the NullHandler has been removed, return!
+    global LOGGING_NULL_HANDLER
+    if LOGGING_NULL_HANDLER is None:
+        # Already removed
         return
 
     root_logger = logging.getLogger()
-    global LOGGING_NULL_HANDLER
 
     for handler in root_logger.handlers:
         if handler is LOGGING_NULL_HANDLER:
@@ -744,8 +982,12 @@ def __remove_queue_logging_handler():
     This function will run once the additional loggers have been synchronized.
     It just removes the QueueLoggingHandler from the logging handlers.
     '''
-    root_logger = logging.getLogger()
     global LOGGING_STORE_HANDLER
+    if LOGGING_STORE_HANDLER is None:
+        # Already removed
+        return
+
+    root_logger = logging.getLogger()
 
     for handler in root_logger.handlers:
         if handler is LOGGING_STORE_HANDLER:
@@ -785,10 +1027,14 @@ def __remove_temp_logging_handler():
 
 def __global_logging_exception_handler(exc_type, exc_value, exc_traceback):
     '''
-    This function will log all python exceptions.
+    This function will log all un-handled python exceptions.
     '''
-    # Do not log the exception or display the traceback on Keyboard Interrupt
-    if exc_type.__name__ != "KeyboardInterrupt":
+    if exc_type.__name__ == "KeyboardInterrupt":
+        # Do not log the exception or display the traceback on Keyboard Interrupt
+        # Stop the logging queue listener thread
+        if is_mp_logging_listener_configured():
+            shutdown_multiprocessing_logging_listener()
+    else:
         # Log the exception
         logging.getLogger(__name__).error(
             'An un-handled exception was caught by salt\'s global exception '
