@@ -12,6 +12,7 @@ from __future__ import absolute_import
 # Import 3rd-party libs
 # pylint: disable=import-error,no-name-in-module,redefined-builtin
 from salt.ext.six.moves.urllib.parse import urljoin as _urljoin
+import salt.ext.six
 import salt.ext.six.moves.http_client
 # pylint: enable=import-error,no-name-in-module
 
@@ -68,6 +69,10 @@ def _query(function,
     base_url = _urljoin(consul_url, '{0}/'.format(api_version))
     url = _urljoin(base_url, function, False)
 
+    if data is None:
+        data = {}
+    data = json.dumps(data)
+
     result = salt.utils.http.query(
         url,
         method=method,
@@ -80,7 +85,7 @@ def _query(function,
     )
 
     if result.get('status', None) == salt.ext.six.moves.http_client.OK:
-        ret['data'] = result['dict']
+        ret['data'] = result.get('dict', result)
         ret['res'] = True
     elif result.get('status', None) == salt.ext.six.moves.http_client.NO_CONTENT:
         ret['res'] = False
@@ -309,7 +314,7 @@ def put(consul_url=None, key=None, value=None, **kwargs):
     ret = _query(consul_url=consul_url,
                  function=function,
                  method=method,
-                 data=json.dumps(data),
+                 data=data,
                  query_params=query_params)
 
     if ret['res']:
@@ -695,9 +700,7 @@ def agent_check_register(consul_url=None, **kwargs):
     if 'name' in kwargs:
         data['Name'] = kwargs['name']
     else:
-        ret['message'] = 'Required parameter "name" is missing.'
-        ret['res'] = False
-        return ret
+        raise SaltInvocationError('Required argument "name" is missing.')
 
     if True not in [True for item in ('script', 'http') if item in kwargs]:
         ret['message'] = 'Required parameter "script" or "http" is missing.'
@@ -971,46 +974,62 @@ def agent_service_register(consul_url=None, **kwargs):
             ret['res'] = False
             return ret
 
-    if 'name' in kwargs:
-        data['Name'] = kwargs['name']
+    lc_kwargs = dict()
+    for k, v in salt.ext.six.iteritems(kwargs):
+        lc_kwargs[k.lower()] = v
 
-    if 'address' in kwargs:
-        data['Address'] = kwargs['address']
+    if 'name' in lc_kwargs:
+        data['Name'] = lc_kwargs['name']
+    else:
+        raise SaltInvocationError('Required argument "name" is missing.')
 
-    if 'port' in kwargs:
-        data['Port'] = kwargs['port']
+    if 'address' in lc_kwargs:
+        data['Address'] = lc_kwargs['address']
 
-    if 'id' in kwargs:
-        data['ID'] = kwargs['id']
+    if 'port' in lc_kwargs:
+        data['Port'] = lc_kwargs['port']
 
-    if 'tags' in kwargs:
-        _tags = kwargs['tags']
+    if 'id' in lc_kwargs:
+        data['ID'] = lc_kwargs['id']
+
+    if 'tags' in lc_kwargs:
+        _tags = lc_kwargs['tags']
         if not isinstance(_tags, list):
             _tags = [_tags]
         data['Tags'] = _tags
 
-    check_elements = ('check_script', 'check_http', 'check_ttl')
-    if True in [True for item in check_elements if item in kwargs]:
-        data['Check'] = {}
+    if 'enabletagoverride' in lc_kwargs:
+        data['EnableTagOverride'] = lc_kwargs['enabletagoverride']
 
-        if 'check_script' in kwargs:
-            if 'interval' not in kwargs:
+    if 'check' in lc_kwargs:
+        dd = dict()
+        for k, v in salt.ext.six.iteritems(lc_kwargs['check']):
+            dd[k.lower()] = v
+        interval_required = False
+        check_dd = dict()
+
+        if 'script' in dd:
+            interval_required = True
+            check_dd['Script'] = dd['script']
+        if 'http' in dd:
+            interval_required = True
+            check_dd['HTTP'] = dd['http']
+        if 'ttl' in dd:
+            check_dd['TTL'] = dd['ttl']
+        if 'interval' in dd:
+            check_dd['Interval'] = dd['interval']
+
+        if interval_required:
+            if 'Interval' not in check_dd:
                 ret['message'] = 'Required parameter "interval" is missing.'
                 ret['res'] = False
                 return ret
-            data['Check']['Script'] = kwargs['check_script']
-            data['Check']['Interval'] = kwargs['check_interval']
+        else:
+            if 'Interval' in check_dd:
+                del check_dd['Interval']  # not required, so ignore it
 
-        if 'check_ttl' in kwargs:
-            data['Check']['TTL'] = kwargs['check_ttl']
-
-        if 'check_http' in kwargs:
-            if 'interval' not in kwargs:
-                ret['message'] = 'Required parameter "interval" is missing.'
-                ret['res'] = False
-                return ret
-            data['Check']['HTTP'] = kwargs['check_http']
-            data['Check']['Interval'] = kwargs['check_interval']
+        if len(check_dd) > 0:
+            data['Check'] = check_dd  # if empty, ignore it
 
     function = 'agent/service/register'
     res = _query(consul_url=consul_url,
@@ -1031,7 +1050,7 @@ def agent_service_deregister(consul_url=None, serviceid=None):
     Used to remove a service.
 
     :param consul_url: The Consul server URL.
-    :param name: A name describing the service.
+    :param serviceid: A serviceid describing the service.
     :return: Boolean and message indicating success or failure.
 
     CLI Example:
@@ -1176,6 +1195,8 @@ def session_create(consul_url=None, **kwargs):
 
     if 'name' in kwargs:
         data['Name'] = kwargs['name']
+    else:
+        raise SaltInvocationError('Required argument "name" is missing.')
 
     if 'checks' in kwargs:
         data['Touch'] = kwargs['touch']
@@ -1193,7 +1214,7 @@ def session_create(consul_url=None, **kwargs):
         if str(_ttl).endswith('s'):
             _ttl = _ttl[:-1]
 
-        if not int(_ttl) >= 0 and not int(_ttl) <= 3600:
+        if int(_ttl) < 0 or int(_ttl) > 3600:
             ret['message'] = ('TTL must be ',
                               'between 0 and 3600.')
             ret['res'] = False
@@ -1455,11 +1476,11 @@ def catalog_register(consul_url=None, **kwargs):
     if res['res']:
         ret['res'] = True
         ret['message'] = ('Catalog registration '
-                          'for {0} successful.'.format(kwargs['name']))
+                          'for {0} successful.'.format(kwargs['node']))
     else:
         ret['res'] = False
         ret['message'] = ('Catalog registration '
-                          'for {0} failed.'.format(kwargs['name']))
+                          'for {0} failed.'.format(kwargs['node']))
     return ret
 
 
@@ -1516,11 +1537,11 @@ def catalog_deregister(consul_url=None, **kwargs):
                  data=data)
     if res['res']:
         ret['res'] = True
-        ret['message'] = 'Catalog item {0} removed.'.format(kwargs['name'])
+        ret['message'] = 'Catalog item {0} removed.'.format(kwargs['node'])
     else:
         ret['res'] = False
         ret['message'] = ('Removing Catalog '
-                          'item {0} failed.'.format(kwargs['name']))
+                          'item {0} failed.'.format(kwargs['node']))
     return ret
 
 
@@ -1980,6 +2001,8 @@ def acl_create(consul_url=None, **kwargs):
 
     if 'name' in kwargs:
         data['Name'] = kwargs['name']
+    else:
+        raise SaltInvocationError('Required argument "name" is missing.')
 
     if 'type' in kwargs:
         data['Type'] = kwargs['type']
@@ -2043,6 +2066,8 @@ def acl_update(consul_url=None, **kwargs):
 
     if 'name' in kwargs:
         data['Name'] = kwargs['name']
+    else:
+        raise SaltInvocationError('Required argument "name" is missing.')
 
     if 'type' in kwargs:
         data['Type'] = kwargs['type']
@@ -2325,6 +2350,8 @@ def event_list(consul_url=None, **kwargs):
 
     if 'name' in kwargs:
         query_params = kwargs['name']
+    else:
+        raise SaltInvocationError('Required argument "name" is missing.')
 
     function = 'event/list/'
     ret = _query(consul_url=consul_url,
