@@ -139,7 +139,7 @@ def _parse_openssl_req(csr_filename):
     '''
     cmd = ('openssl req -text -noout -in {0}'.format(csr_filename))
 
-    output = __salt__['cmd.run_stderr'](cmd)
+    output = __salt__['cmd.run_stdout'](cmd)
 
     output = re.sub(r': rsaEncryption', ':', output)
     output = re.sub(r'[0-9a-f]{2}:', '', output)
@@ -159,14 +159,12 @@ def _get_csr_extensions(csr):
     csrtempfile.flush()
     csryaml = _parse_openssl_req(csrtempfile.name)
     csrtempfile.close()
-    try:
+    if csryaml and 'Requested Extensions' in csryaml['Certificate Request']['Data']:
         csrexts = csryaml['Certificate Request']['Data']['Requested Extensions']
-    except TypeError:
-        csrexts = {}
 
-    for short_name, long_name in six.iteritems(EXT_NAME_MAPPINGS):
-        if long_name in csrexts:
-            ret[short_name] = csrexts[long_name]
+        for short_name, long_name in six.iteritems(EXT_NAME_MAPPINGS):
+            if long_name in csrexts:
+                ret[short_name] = csrexts[long_name]
 
     return ret
 
@@ -179,7 +177,7 @@ def _parse_openssl_crl(crl_filename):
     '''
     cmd = ('openssl crl -text -noout -in {0}'.format(crl_filename))
 
-    output = __salt__['cmd.run_stderr'](cmd)
+    output = __salt__['cmd.run_stdout'](cmd)
 
     crl = {}
     for line in output.split('\n'):
@@ -267,7 +265,8 @@ def _text_or_file(input_):
     Determines if input is a path to a file, or a string with the content to be parsed.
     '''
     if os.path.isfile(input_):
-        return salt.utils.fopen(input_).read()
+        with salt.utils.fopen(input_) as fp_:
+            return fp_.read()
     else:
         return input_
 
@@ -626,8 +625,11 @@ def write_pem(text, path, pem_type=None):
 
         salt '*' x509.write_pem "-----BEGIN CERTIFICATE-----MIIGMzCCBBugA..." path=/etc/pki/mycert.crt
     '''
+    old_umask = os.umask(0o77)
     text = get_pem_entry(text, pem_type=pem_type)
-    salt.utils.fopen(path, 'w').write(text)
+    with salt.utils.fopen(path, 'w') as fp_:
+        fp_.write(text)
+    os.umask(old_umask)
     return 'PEM written to {0}'.format(path)
 
 
@@ -954,7 +956,7 @@ def create_certificate(path=None, text=False, ca_server=None, **kwargs):
 
     extensions:
         The following arguments set X509v3 Extension values. If the value starts with ``critical ``,
-        the extension will be marked as critical
+        the extension will be marked as critical.
 
         Some special extensions are ``subjectKeyIdentifier`` and ``authorityKeyIdentifier``.
 
@@ -1117,7 +1119,6 @@ def create_certificate(path=None, text=False, ca_server=None, **kwargs):
             kwargs[prop] = default
 
     cert = M2Crypto.X509.X509()
-    subject = cert.get_subject()
 
     # X509 Version 3 has a value of 2 in the field.
     # Version 2 has a value of 1.
@@ -1145,11 +1146,13 @@ def create_certificate(path=None, text=False, ca_server=None, **kwargs):
     if 'csr' in kwargs:
         kwargs['public_key'] = kwargs['csr']
         csr = _get_request_obj(kwargs['csr'])
-        subject = csr.get_subject()
+        cert.set_subject(csr.get_subject())
         csrexts = read_csr(kwargs['csr'])['X509v3 Extensions']
 
     cert.set_pubkey(get_public_key(kwargs['public_key'], asObj=True))
 
+    subject = cert.get_subject()
+
     for entry, num in six.iteritems(subject.nid):                  # pylint: disable=unused-variable
         if entry in kwargs:
             setattr(subject, entry, kwargs[entry])
@@ -1356,7 +1359,7 @@ def verify_crl(crl, cert):
 
     cmd = ('openssl crl -noout -in {0} -CAfile {1}'.format(crltempfile.name, certtempfile.name))
 
-    output = __salt__['cmd.run_stderr'](cmd)
+    output = __salt__['cmd.run_stdout'](cmd)
 
     crltempfile.close()
     certtempfile.close()
