@@ -349,9 +349,13 @@ def list_jobs(ext_source=None,
         if search_target and _match:
             _match = False
             if 'Target' in ret[item]:
-                for key in salt.utils.split_input(search_target):
-                    if fnmatch.fnmatch(ret[item]['Target'], key):
-                        _match = True
+                targets = ret[item]['Target']
+                if isinstance(targets, six.string_types):
+                    targets = [targets]
+                for target in targets:
+                    for key in salt.utils.split_input(search_target):
+                        if fnmatch.fnmatch(target, key):
+                            _match = True
 
         if search_function and _match:
             _match = False
@@ -370,7 +374,7 @@ def list_jobs(ext_source=None,
             else:
                 log.error(
                     '\'dateutil\' library not available, skipping start_time '
-                    'comparision.'
+                    'comparison.'
                 )
 
         if end_time and _match:
@@ -383,7 +387,7 @@ def list_jobs(ext_source=None,
             else:
                 log.error(
                     '\'dateutil\' library not available, skipping end_time '
-                    'comparision.'
+                    'comparison.'
                 )
 
         if _match:
@@ -488,6 +492,38 @@ def print_job(jid, ext_source=None, outputter=None):
         return ret
 
 
+def exit_success(jid, ext_source=None):
+    '''
+    Check if a job has been executed and exit successfully
+
+    jid
+        The jid to look up.
+    ext_source
+        The external job cache to use. Default: `None`.
+
+    CLI Example:
+    .. code-block:: bash
+        salt-run jobs.exit_success 20160520145827701627
+    '''
+    ret = dict()
+
+    data = list_job(
+        jid,
+        ext_source=ext_source
+    )
+
+    minions = data['Minions']
+    result = data['Result']
+
+    for minion in minions:
+        if minion in result and 'return' in result[minion]:
+            ret[minion] = True if result[minion]['return'] else False
+        else:
+            ret[minion] = False
+
+    return ret
+
+
 def last_run(ext_source=None,
              outputter=None,
              metadata=None,
