@@ -16,7 +16,7 @@ For remote signing, peers must be permitted to remotely call the
 :mod:`sign_remote_certificate <salt.modules.x509.sign_remote_certificate>` function.
 
 
-/etc/salt/master.d/peer.sls
+/etc/salt/master.d/peer.conf
 
 .. code-block:: yaml
 
@@ -205,7 +205,7 @@ def private_key_managed(name,
                         new=False,
                         backup=False):
     '''
-    Manage a private key's existance.
+    Manage a private key's existence.
 
     name:
         Path to the private key
@@ -347,11 +347,11 @@ def certificate_managed(name,
         Path to the certificate
 
     days_remaining:
-        The minimum number of days remaining when the certificate should be recreted. Default is 90. A
+        The minimum number of days remaining when the certificate should be recreated. Default is 90. A
         value of 0 disables automatic renewal.
 
     backup:
-        When replacing an existing file, backup the old file onthe minion. Default is False.
+        When replacing an existing file, backup the old file on the minion. Default is False.
 
     kwargs:
         Any arguments supported by :mod:`x509.create_certificate <salt.modules.x509.create_certificate>`
@@ -392,6 +392,9 @@ def certificate_managed(name,
     '''
     ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}
 
+    if 'path' in kwargs:
+        name = kwargs.pop('path')
+
     current_days_remaining = 0
     current_comp = {}
 
@@ -614,16 +617,20 @@ def pem_managed(name,
 
     new = __salt__['x509.get_pem_entry'](text=text)
 
-    if os.path.isfile(name):
-        current = salt.utils.fopen(name).read()
-    else:
-        current = '{0} does not exist.'.format(name)
+    try:
+        with salt.utils.fopen(name) as fp_:
+            current = fp_.read()
+    except (OSError, IOError):
+        current = '{0} does not exist or is unreadable'.format(name)
 
     if new == current:
         ret['result'] = True
         ret['comment'] = 'The file is already in the correct state'
         return ret
 
+    ret['changes']['new'] = new
+    ret['changes']['old'] = current
+
     if __opts__['test'] is True:
         ret['result'] = None
         ret['comment'] = 'The file {0} will be updated.'.format(name)
