@@ -55,37 +55,31 @@ from __future__ import absolute_import
 # Import python libs
 import os
 import time
-import errno
-import signal
 import fnmatch
 import hashlib
 import logging
 import datetime
-import multiprocessing
 from collections import MutableMapping
+from multiprocessing.util import Finalize
 
 # Import third party libs
 import salt.ext.six as six
-try:
-    import zmq
-    import zmq.eventloop.ioloop
-    # support pyzmq 13.0.x, TODO: remove once we force people to 14.0.x
-    if not hasattr(zmq.eventloop.ioloop, 'ZMQIOLoop'):
-        zmq.eventloop.ioloop.ZMQIOLoop = zmq.eventloop.ioloop.IOLoop
-    import zmq.eventloop.zmqstream
-except ImportError:
-    # Local mode does not need zmq
-    pass
+import tornado.ioloop
+import tornado.iostream
 
 # Import salt libs
 import salt.config
 import salt.payload
 import salt.loader
 import salt.utils
+import salt.utils.async
 import salt.utils.cache
 import salt.utils.dicttrim
 import salt.utils.process
 import salt.utils.zeromq
+import salt.log.setup
+import salt.defaults.exitcodes
+import salt.transport.ipc
 
 log = logging.getLogger(__name__)
 
@@ -116,16 +110,23 @@ TAGS = {
 }
 
 
-def get_event(node, sock_dir=None, transport='zeromq', opts=None, listen=True):
+def get_event(
+        node, sock_dir=None, transport='zeromq',
+        opts=None, listen=True, io_loop=None):
     '''
     Return an event object suitable for the named transport
+
+    :param IOLoop io_loop: Pass in an io_loop if you want asynchronous
+                           operation for obtaining events. Eg use of
+                           set_event_handler() API. Otherwise, operation
+                           will be synchronous.
     '''
     sock_dir = sock_dir or opts['sock_dir']
     # TODO: AIO core is separate from transport
     if transport in ('zeromq', 'tcp'):
         if node == 'master':
-            return MasterEvent(sock_dir, opts, listen=listen)
-        return SaltEvent(node, sock_dir, opts, listen=listen)
+            return MasterEvent(sock_dir, opts, listen=listen, io_loop=io_loop)
+        return SaltEvent(node, sock_dir, opts, listen=listen, io_loop=io_loop)
     elif transport == 'raet':
         import salt.utils.raetevent
         return salt.utils.raetevent.RAETEvent(node,
@@ -134,13 +135,13 @@ def get_event(node, sock_dir=None, transport='zeromq', opts=None, listen=True):
                                               opts=opts)
 
 
-def get_master_event(opts, sock_dir, listen=True):
+def get_master_event(opts, sock_dir, listen=True, io_loop=None):
     '''
     Return an event object suitable for the named transport
     '''
     # TODO: AIO core is separate from transport
     if opts['transport'] in ('zeromq', 'tcp'):
-        return MasterEvent(sock_dir, opts, listen=listen)
+        return MasterEvent(sock_dir, opts, listen=listen, io_loop=io_loop)
     elif opts['transport'] == 'raet':
         import salt.utils.raetevent
         return salt.utils.raetevent.MasterEvent(
@@ -174,12 +175,26 @@ class SaltEvent(object):
     RAET compatible
     The base class used to manage salt events
     '''
-    def __init__(self, node, sock_dir=None, opts=None, listen=True):
+    def __init__(
+            self, node, sock_dir=None,
+            opts=None, listen=True, io_loop=None):
+        '''
+        :param IOLoop io_loop: Pass in an io_loop if you want asynchronous
+                               operation for obtaining events. Eg use of
+                               set_event_handler() API. Otherwise, operation
+                               will be synchronous.
+        '''
         self.serial = salt.payload.Serial({'serial': 'msgpack'})
-        self.context = zmq.Context()
-        self.poller = zmq.Poller()
+        if io_loop is not None:
+            self.io_loop = io_loop
+            self._run_io_loop_sync = False
+        else:
+            self.io_loop = tornado.ioloop.IOLoop()
+            self._run_io_loop_sync = True
         self.cpub = False
         self.cpush = False
+        self.subscriber = None
+        self.pusher = None
 
         if opts is None:
             opts = {}
@@ -194,14 +209,18 @@ class SaltEvent(object):
         else:
             self.opts['sock_dir'] = sock_dir
 
-        if salt.utils.is_windows() and not hasattr(self.opts, 'ipc_mode'):
+        if salt.utils.is_windows() and 'ipc_mode' not in opts:
             self.opts['ipc_mode'] = 'tcp'
         self.puburi, self.pulluri = self.__load_uri(sock_dir, node)
         self.pending_tags = []
         self.pending_events = []
-        if not self.cpub:
-            self.connect_pub()
         self.__load_cache_regex()
+        if listen and not self.cpub:
+            # Only connect to the publisher at initialization time if
+            # we know we want to listen. If we connect to the publisher
+            # and don't read out events from the buffer on an on-going basis,
+            # the buffer will grow resulting in big memory usage.
+            self.connect_pub()
 
     @classmethod
     def __load_cache_regex(cls):
@@ -222,46 +241,34 @@ class SaltEvent(object):
         '''
         if node == 'master':
             if self.opts['ipc_mode'] == 'tcp':
-                puburi = 'tcp://127.0.0.1:{0}'.format(
-                    self.opts['tcp_master_pub_port']
-                    )
-                pulluri = 'tcp://127.0.0.1:{0}'.format(
-                    self.opts['tcp_master_pull_port']
-                    )
+                puburi = int(self.opts['tcp_master_pub_port'])
+                pulluri = int(self.opts['tcp_master_pull_port'])
             else:
-                puburi = 'ipc://{0}'.format(os.path.join(
+                puburi = os.path.join(
                     sock_dir,
                     'master_event_pub.ipc'
-                    ))
-                salt.utils.zeromq.check_ipc_path_max_len(puburi)
-                pulluri = 'ipc://{0}'.format(os.path.join(
+                )
+                pulluri = os.path.join(
                     sock_dir,
                     'master_event_pull.ipc'
-                    ))
-                salt.utils.zeromq.check_ipc_path_max_len(pulluri)
+                )
         else:
             if self.opts['ipc_mode'] == 'tcp':
-                puburi = 'tcp://127.0.0.1:{0}'.format(
-                    self.opts['tcp_pub_port']
-                    )
-                pulluri = 'tcp://127.0.0.1:{0}'.format(
-                    self.opts['tcp_pull_port']
-                    )
+                puburi = int(self.opts['tcp_pub_port'])
+                pulluri = int(self.opts['tcp_pull_port'])
             else:
                 hash_type = getattr(hashlib, self.opts['hash_type'])
                 # Only use the first 10 chars to keep longer hashes from exceeding the
                 # max socket path length.
                 id_hash = hash_type(salt.utils.to_bytes(self.opts['id'])).hexdigest()[:10]
-                puburi = 'ipc://{0}'.format(os.path.join(
+                puburi = os.path.join(
                     sock_dir,
                     'minion_event_{0}_pub.ipc'.format(id_hash)
-                    ))
-                salt.utils.zeromq.check_ipc_path_max_len(puburi)
-                pulluri = 'ipc://{0}'.format(os.path.join(
+                )
+                pulluri = os.path.join(
                     sock_dir,
                     'minion_event_{0}_pull.ipc'.format(id_hash)
-                    ))
-                salt.utils.zeromq.check_ipc_path_max_len(pulluri)
+                )
         log.debug(
             '{0} PUB socket URI: {1}'.format(self.__class__.__name__, puburi)
         )
@@ -301,32 +308,69 @@ class SaltEvent(object):
             if any(pmatch_func(evt['tag'], ptag) for ptag, pmatch_func in self.pending_tags):
                 self.pending_events.append(evt)
 
-    def connect_pub(self):
+    def connect_pub(self, timeout=None):
         '''
         Establish the publish connection
         '''
-        self.sub = self.context.socket(zmq.SUB)
-        try:
-            self.sub.setsockopt(zmq.HWM, self.opts['salt_event_pub_hwm'])
-        except AttributeError:
-            self.sub.setsockopt(zmq.SNDHWM, self.opts['salt_event_pub_hwm'])
-            self.sub.setsockopt(zmq.RCVHWM, self.opts['salt_event_pub_hwm'])
-        self.sub.connect(self.puburi)
-        self.poller.register(self.sub, zmq.POLLIN)
-        self.sub.setsockopt_string(zmq.SUBSCRIBE, u'')
-        self.sub.setsockopt(zmq.LINGER, 5000)
-        self.cpub = True
+        if self.cpub:
+            return True
 
-    def connect_pull(self, timeout=1000):
+        if self._run_io_loop_sync:
+            with salt.utils.async.current_ioloop(self.io_loop):
+                if self.subscriber is None:
+                    self.subscriber = salt.transport.ipc.IPCMessageSubscriber(
+                    self.puburi,
+                    io_loop=self.io_loop
+                )
+                try:
+                    self.io_loop.run_sync(
+                        lambda: self.subscriber.connect(timeout=timeout))
+                    self.cpub = True
+                except Exception:
+                    pass
+        else:
+            if self.subscriber is None:
+                self.subscriber = salt.transport.ipc.IPCMessageSubscriber(
+                self.puburi,
+                io_loop=self.io_loop
+            )
+
+            # For the async case, the connect will be defered to when
+            # set_event_handler() is invoked.
+            self.cpub = True
+        return self.cpub
+
+    def connect_pull(self, timeout=1):
         '''
         Establish a connection with the event pull socket
-        Set the linger timeout of the socket options to timeout (in milliseconds)
-        Default timeout is 1000 ms
+        Default timeout is 1 s
         '''
-        self.push = self.context.socket(zmq.PUSH)
-        self.push.setsockopt(zmq.LINGER, timeout)
-        self.push.connect(self.pulluri)
-        self.cpush = True
+        if self.cpush:
+            return True
+
+        if self._run_io_loop_sync:
+            with salt.utils.async.current_ioloop(self.io_loop):
+                if self.pusher is None:
+                    self.pusher = salt.transport.ipc.IPCMessageClient(
+                        self.pulluri,
+                        io_loop=self.io_loop
+                    )
+                try:
+                    self.io_loop.run_sync(
+                        lambda: self.pusher.connect(timeout=timeout))
+                    self.cpush = True
+                except Exception:
+                    pass
+        else:
+            if self.pusher is None:
+                self.pusher = salt.transport.ipc.IPCMessageClient(
+                    self.pulluri,
+                    io_loop=self.io_loop
+                )
+            # For the async case, the connect will be defered to when
+            # fire_event() is invoked.
+            self.cpush = True
+        return self.cpush
 
     @classmethod
     def unpack(cls, raw, serial=None):
@@ -421,6 +465,10 @@ class SaltEvent(object):
         run_once = False
         if no_block is True:
             wait = 0
+        elif wait == 0:
+            # If no_block is False and wait is 0, that
+            # means an infinite timeout.
+            wait = None
         while (run_once is False and not wait) or time.time() <= timeout_at:
             if no_block is True:
                 if run_once is True:
@@ -428,17 +476,20 @@ class SaltEvent(object):
                 # Trigger that at least a single iteration has gone through
                 run_once = True
             try:
-                # convert to milliseconds
-                socks = dict(self.poller.poll(wait * 1000))
-                if socks.get(self.sub) != zmq.POLLIN:
-                    continue
+                # tornado.ioloop.IOLoop.run_sync() timeouts are in seconds.
+                # IPCMessageSubscriber.read_sync() uses this type of timeout.
+                if not self.cpub and not self.connect_pub(timeout=wait):
+                    break
 
-                ret = self.get_event_block()
-            except zmq.ZMQError as ex:
-                if ex.errno == errno.EAGAIN or ex.errno == errno.EINTR:
-                    continue
-                else:
-                    raise
+                raw = self.subscriber.read_sync(timeout=wait)
+                if raw is None:
+                    break
+                mtag, data = self.unpack(raw, self.serial)
+                ret = {'data': data, 'tag': mtag}
+            except KeyboardInterrupt:
+                return {'tag': 'salt/event/exit', 'data': {}}
+            except (tornado.iostream.StreamClosedError, RuntimeError):
+                return None
 
             if not match_func(ret['tag'], tag):
                 # tag not match
@@ -451,7 +502,7 @@ class SaltEvent(object):
 
             log.trace('get_event() received = {0}'.format(ret))
             return ret
-        log.trace('_get_event() waited {0} seconds and received nothing'.format(wait * 1000))
+        log.trace('_get_event() waited {0} seconds and received nothing'.format(wait))
         return None
 
     def get_event(self,
@@ -506,6 +557,8 @@ class SaltEvent(object):
         request, it MUST subscribe the result to ensure the response is not lost
         should other regions of code call get_event for other purposes.
         '''
+        assert self._run_io_loop_sync
+
         if use_pending is not None:
             salt.utils.warn_until(
                 'Nitrogen',
@@ -522,7 +575,8 @@ class SaltEvent(object):
 
         ret = self._check_pending(tag, match_func)
         if ret is None:
-            ret = self._get_event(wait, tag, match_func, no_block)
+            with salt.utils.async.current_ioloop(self.io_loop):
+                ret = self._get_event(wait, tag, match_func, no_block)
 
         if ret is None or full:
             return ret
@@ -532,9 +586,14 @@ class SaltEvent(object):
     def get_event_noblock(self):
         '''Get the raw event without blocking or any other niceties
         '''
+        assert self._run_io_loop_sync
+
         if not self.cpub:
-            self.connect_pub()
-        raw = self.sub.recv(zmq.NOBLOCK)
+            if not self.connect_pub():
+                return None
+        raw = self.subscriber.read_sync(timeout=0)
+        if raw is None:
+            return None
         mtag, data = self.unpack(raw, self.serial)
         return {'data': data, 'tag': mtag}
 
@@ -542,7 +601,14 @@ class SaltEvent(object):
         '''Get the raw event in a blocking fashion
            Slower, but decreases the possibility of dropped events
         '''
-        raw = self.sub.recv()
+        assert self._run_io_loop_sync
+
+        if not self.cpub:
+            if not self.connect_pub():
+                return None
+        raw = self.subscriber.read_sync(timeout=None)
+        if raw is None:
+            return None
         mtag, data = self.unpack(raw, self.serial)
         return {'data': data, 'tag': mtag}
 
@@ -562,16 +628,22 @@ class SaltEvent(object):
         event identifier "tag"
 
         The default is 1000 ms
-        Note the linger timeout must be at least as long as this timeout
         '''
         if not str(tag):  # no empty tags allowed
             raise ValueError('Empty tag.')
 
         if not isinstance(data, MutableMapping):  # data must be dict
-            raise ValueError('Dict object expected, not "{0!r}".'.format(data))
+            raise ValueError(
+                'Dict object expected, not \'{0}\'.'.format(data)
+            )
 
         if not self.cpush:
-            self.connect_pull(timeout=timeout)
+            if timeout is not None:
+                timeout_s = float(timeout) / 1000
+            else:
+                timeout_s = None
+            if not self.connect_pull(timeout=timeout_s):
+                return False
 
         data['_stamp'] = datetime.datetime.utcnow().isoformat()
 
@@ -583,11 +655,16 @@ class SaltEvent(object):
         )
         log.debug('Sending event - data = {0}'.format(data))
         event = '{0}{1}{2}'.format(tag, tagend, serialized_data)
-        try:
-            self.push.send(salt.utils.to_bytes(event, 'utf-8'))
-        except Exception as ex:
-            log.debug(ex)
-            raise
+        msg = salt.utils.to_bytes(event, 'utf-8')
+        if self._run_io_loop_sync:
+            with salt.utils.async.current_ioloop(self.io_loop):
+                try:
+                    self.io_loop.run_sync(lambda: self.pusher.send(msg))
+                except Exception as ex:
+                    log.debug(ex)
+                    raise
+        else:
+            self.io_loop.spawn_callback(self.pusher.send, msg)
         return True
 
     def fire_master(self, data, tag, timeout=1000):
@@ -605,39 +682,13 @@ class SaltEvent(object):
         }
         return self.fire_event(msg, "fire_master", timeout)
 
-    def destroy(self, linger=5000):
-        if self.cpub is True and self.sub.closed is False:
-            # Wait at most 2.5 secs to send any remaining messages in the
-            # socket or the context.term() below will hang indefinitely.
-            # See https://github.com/zeromq/pyzmq/issues/102
-            self.sub.close()
-        if self.cpush is True and self.push.closed is False:
-            self.push.close()
-        # If sockets are not unregistered from a poller, nothing which touches
-        # that poller gets garbage collected. The Poller itself, its
-        # registered sockets and the Context
-        if isinstance(self.poller.sockets, dict):
-            for socket in six.iterkeys(self.poller.sockets):
-                if socket.closed is False:
-                    socket.setsockopt(zmq.LINGER, linger)
-                    socket.close()
-                self.poller.unregister(socket)
-        else:
-            for socket in self.poller.sockets:
-                if socket[0].closed is False:
-                    socket[0].setsockopt(zmq.LINGER, linger)
-                    socket[0].close()
-                self.poller.unregister(socket[0])
-        if self.context.closed is False:
-            self.context.term()
-
-        # Hardcore destruction
-        if hasattr(self.context, 'destroy'):
-            self.context.destroy(linger=1)
-
-        # https://github.com/zeromq/pyzmq/issues/173#issuecomment-4037083
-        # Assertion failed: get_load () == 0 (poller_base.cpp:32)
-        time.sleep(0.025)
+    def destroy(self):
+        if self.subscriber is not None:
+            self.subscriber.close()
+        if self.pusher is not None:
+            self.pusher.close()
+        if self._run_io_loop_sync:
+            self.io_loop.close()
 
     def fire_ret_load(self, load):
         '''
@@ -673,6 +724,17 @@ class SaltEvent(object):
                 except Exception:
                     pass
 
+    def set_event_handler(self, event_handler):
+        '''
+        Invoke the event_handler callback each time an event arrives.
+        '''
+        assert not self._run_io_loop_sync
+
+        if not self.cpub:
+            self.connect_pub()
+        # This will handle reconnects
+        self.subscriber.read_async(event_handler)
+
     def __del__(self):
         # skip exceptions in destroy-- since destroy() doesn't cover interpreter
         # shutdown-- where globals start going missing
@@ -688,8 +750,9 @@ class MasterEvent(SaltEvent):
     RAET compatible
     Create a master event management object
     '''
-    def __init__(self, sock_dir, opts=None, listen=True):
-        super(MasterEvent, self).__init__('master', sock_dir, opts, listen=listen)
+    def __init__(self, sock_dir, opts=None, listen=True, io_loop=None):
+        super(MasterEvent, self).__init__(
+            'master', sock_dir, opts, listen=listen, io_loop=io_loop)
 
 
 class LocalClientEvent(MasterEvent):
@@ -722,9 +785,10 @@ class MinionEvent(SaltEvent):
     RAET compatible
     Create a master event management object
     '''
-    def __init__(self, opts, listen=True):
+    def __init__(self, opts, listen=True, io_loop=None):
         super(MinionEvent, self).__init__(
-            'minion', sock_dir=opts.get('sock_dir'), opts=opts, listen=listen)
+            'minion', sock_dir=opts.get('sock_dir'),
+            opts=opts, listen=listen, io_loop=io_loop)
 
 
 class AsyncEventPublisher(object):
@@ -735,12 +799,13 @@ class AsyncEventPublisher(object):
     '''
     def __init__(self, opts, publish_handler, io_loop=None):
         self.opts = salt.config.DEFAULT_MINION_OPTS.copy()
+        default_minion_sock_dir = self.opts['sock_dir']
         self.opts.update(opts)
 
         self.publish_handler = publish_handler
 
-        self.io_loop = io_loop or zmq.eventloop.ioloop.ZMQIOLoop()
-        self.context = zmq.Context()
+        self.io_loop = io_loop or tornado.ioloop.IOLoop.current()
+        self._closing = False
 
         hash_type = getattr(hashlib, self.opts['hash_type'])
         # Only use the first 10 chars to keep longer hashes from exceeding the
@@ -759,20 +824,12 @@ class AsyncEventPublisher(object):
         if os.path.exists(epull_sock_path):
             os.unlink(epull_sock_path)
 
-        self.epub_sock = self.context.socket(zmq.PUB)
-
         if self.opts['ipc_mode'] == 'tcp':
-            epub_uri = 'tcp://127.0.0.1:{0}'.format(
-                self.opts['tcp_pub_port']
-            )
-            epull_uri = 'tcp://127.0.0.1:{0}'.format(
-                self.opts['tcp_pull_port']
-            )
+            epub_uri = int(self.opts['tcp_pub_port'])
+            epull_uri = int(self.opts['tcp_pull_port'])
         else:
-            epub_uri = 'ipc://{0}'.format(epub_sock_path)
-            salt.utils.zeromq.check_ipc_path_max_len(epub_uri)
-            epull_uri = 'ipc://{0}'.format(epull_sock_path)
-            salt.utils.zeromq.check_ipc_path_max_len(epull_uri)
+            epub_uri = epub_sock_path
+            epull_uri = epull_sock_path
 
         log.debug(
             '{0} PUB socket URI: {1}'.format(
@@ -785,12 +842,7 @@ class AsyncEventPublisher(object):
             )
         )
 
-        # Check to make sure the sock_dir is available, create if not
-        default_minion_sock_dir = os.path.join(
-            salt.syspaths.SOCK_DIR,
-            'minion'
-        )
-        minion_sock_dir = self.opts.get('sock_dir', default_minion_sock_dir)
+        minion_sock_dir = self.opts['sock_dir']
 
         if not os.path.isdir(minion_sock_dir):
             # Let's try to create the directory defined on the configuration
@@ -812,149 +864,175 @@ class AsyncEventPublisher(object):
                         # Let's stop at this stage
                         raise
 
-        # Create the pull socket
-        self.epull_sock = self.context.socket(zmq.PULL)
+        self.publisher = salt.transport.ipc.IPCMessagePublisher(
+            self.opts,
+            epub_uri,
+            io_loop=self.io_loop
+        )
 
-        # Securely bind the event sockets
-        if self.opts['ipc_mode'] != 'tcp':
-            old_umask = os.umask(0o177)
+        self.puller = salt.transport.ipc.IPCMessageServer(
+            epull_uri,
+            io_loop=self.io_loop,
+            payload_handler=self.handle_publish
+        )
+
+        log.info('Starting pull socket on {0}'.format(epull_uri))
+        old_umask = os.umask(0o177)
         try:
-            log.info('Starting pub socket on {0}'.format(epub_uri))
-            self.epub_sock.bind(epub_uri)
-            log.info('Starting pull socket on {0}'.format(epull_uri))
-            self.epull_sock.bind(epull_uri)
+            self.publisher.start()
+            self.puller.start()
         finally:
-            if self.opts['ipc_mode'] != 'tcp':
-                os.umask(old_umask)
+            os.umask(old_umask)
 
-        self.stream = zmq.eventloop.zmqstream.ZMQStream(self.epull_sock, io_loop=self.io_loop)
-        self.stream.on_recv(self.handle_publish)
-
-    def handle_publish(self, package):
+    def handle_publish(self, package, _):
         '''
         Get something from epull, publish it out epub, and return the package (or None)
         '''
-        package = package[0]
         try:
-            self.epub_sock.send(package)
+            self.publisher.publish(package)
             self.io_loop.spawn_callback(self.publish_handler, package)
             return package
         # Add an extra fallback in case a forked process leeks through
-        except zmq.ZMQError as exc:
-            # The interrupt caused by python handling the
-            # SIGCHLD. Throws this error with errno == EINTR.
-            # Nothing to receive on the zmq socket throws this error
-            # with EAGAIN.
-            # Both are safe to ignore
-            if exc.errno != errno.EAGAIN and exc.errno != errno.EINTR:
-                log.critical('Unexpected ZMQError while polling minion',
-                             exc_info=True)
+        except Exception:
+            log.critical('Unexpected error while polling minion events',
+                         exc_info=True)
             return None
 
-    def destroy(self):
-        if hasattr(self, 'stream') and self.stream.closed is False:
-            self.stream.close()
-        if hasattr(self, 'epub_sock') and self.epub_sock.closed is False:
-            self.epub_sock.close()
-        if hasattr(self, 'epull_sock') and self.epull_sock.closed is False:
-            self.epull_sock.close()
-        if hasattr(self, 'context') and self.context.closed is False:
-            self.context.term()
+    def close(self):
+        if self._closing:
+            return
+        self._closing = True
+        if hasattr(self, 'publisher'):
+            self.publisher.close()
+        if hasattr(self, 'puller'):
+            self.puller.close()
 
     def __del__(self):
-        self.destroy()
+        self.close()
 
 
-class EventPublisher(multiprocessing.Process):
+class EventPublisher(salt.utils.process.SignalHandlingMultiprocessingProcess):
     '''
     The interface that takes master events and republishes them out to anyone
     who wants to listen
     '''
-    def __init__(self, opts):
-        super(EventPublisher, self).__init__()
+    def __init__(self, opts, log_queue=None):
+        super(EventPublisher, self).__init__(log_queue=log_queue)
         self.opts = salt.config.DEFAULT_MASTER_OPTS.copy()
         self.opts.update(opts)
+        self._closing = False
+
+    # __setstate__ and __getstate__ are only used on Windows.
+    # We do this so that __init__ will be invoked on Windows in the child
+    # process so that a register_after_fork() equivalent will work on Windows.
+    def __setstate__(self, state):
+        self._is_child = True
+        self.__init__(state['opts'], log_queue=state['log_queue'])
+
+    def __getstate__(self):
+        return {'opts': self.opts,
+                'log_queue': self.log_queue}
 
     def run(self):
         '''
         Bind the pub and pull sockets for events
         '''
         salt.utils.appendproctitle(self.__class__.__name__)
-        linger = 5000
-        # Set up the context
-        self.context = zmq.Context(1)
-        # Prepare the master event publisher
-        self.epub_sock = self.context.socket(zmq.PUB)
-        try:
-            self.epub_sock.setsockopt(zmq.HWM, self.opts['event_publisher_pub_hwm'])
-        except AttributeError:
-            self.epub_sock.setsockopt(zmq.SNDHWM, self.opts['event_publisher_pub_hwm'])
-            self.epub_sock.setsockopt(zmq.RCVHWM, self.opts['event_publisher_pub_hwm'])
-        # Prepare master event pull socket
-        self.epull_sock = self.context.socket(zmq.PULL)
-        if self.opts['ipc_mode'] == 'tcp':
-            epub_uri = 'tcp://127.0.0.1:{0}'.format(
-                self.opts['tcp_master_pub_port']
+        self.io_loop = tornado.ioloop.IOLoop()
+        with salt.utils.async.current_ioloop(self.io_loop):
+            if self.opts['ipc_mode'] == 'tcp':
+                epub_uri = int(self.opts['tcp_master_pub_port'])
+                epull_uri = int(self.opts['tcp_master_pull_port'])
+            else:
+                epub_uri = os.path.join(
+                    self.opts['sock_dir'],
+                    'master_event_pub.ipc'
                 )
-            epull_uri = 'tcp://127.0.0.1:{0}'.format(
-                self.opts['tcp_master_pull_port']
+                epull_uri = os.path.join(
+                    self.opts['sock_dir'],
+                    'master_event_pull.ipc'
                 )
-        else:
-            epub_uri = 'ipc://{0}'.format(
-                os.path.join(self.opts['sock_dir'], 'master_event_pub.ipc')
-                )
-            salt.utils.zeromq.check_ipc_path_max_len(epub_uri)
-            epull_uri = 'ipc://{0}'.format(
-                os.path.join(self.opts['sock_dir'], 'master_event_pull.ipc')
-                )
-            salt.utils.zeromq.check_ipc_path_max_len(epull_uri)
 
-        # Start the master event publisher
-        old_umask = os.umask(0o177)
+            self.publisher = salt.transport.ipc.IPCMessagePublisher(
+                self.opts,
+                epub_uri,
+                io_loop=self.io_loop
+            )
+
+            self.puller = salt.transport.ipc.IPCMessageServer(
+                epull_uri,
+                io_loop=self.io_loop,
+                payload_handler=self.handle_publish,
+            )
+
+            # Start the master event publisher
+            old_umask = os.umask(0o177)
+            try:
+                self.publisher.start()
+                self.puller.start()
+                if self.opts['client_acl'] or self.opts['client_acl_blacklist']:
+                    salt.utils.warn_until(
+                            'Nitrogen',
+                            'ACL rules should be configured with \'publisher_acl\' and '
+                            '\'publisher_acl_blacklist\' not \'client_acl\' and '
+                            '\'client_acl_blacklist\'. This functionality will be removed in Salt '
+                            'Nitrogen.'
+                            )
+                if (self.opts['ipc_mode'] != 'tcp' and (
+                        self.opts['publisher_acl'] or
+                        self.opts['client_acl'] or
+                        self.opts['external_auth'])):
+                    os.chmod(os.path.join(
+                        self.opts['sock_dir'], 'master_event_pub.ipc'), 0o666)
+            finally:
+                os.umask(old_umask)
+
+            # Make sure the IO loop and respective sockets are closed and
+            # destroyed
+            Finalize(self, self.close, exitpriority=15)
+
+            self.io_loop.start()
+
+    def handle_publish(self, package, _):
+        '''
+        Get something from epull, publish it out epub, and return the package (or None)
+        '''
         try:
-            self.epull_sock.bind(epull_uri)
-            self.epub_sock.bind(epub_uri)
-            if (self.opts['ipc_mode'] != 'tcp' and (
-                    self.opts['client_acl'] or
-                    self.opts['external_auth'])):
-                os.chmod(os.path.join(
-                    self.opts['sock_dir'], 'master_event_pub.ipc'), 0o666)
-        finally:
-            os.umask(old_umask)
-        try:
-            while True:
-                # Catch and handle EINTR from when this process is sent
-                # SIGUSR1 gracefully so we don't choke and die horribly
-                try:
-                    package = self.epull_sock.recv()
-                    self.epub_sock.send(package)
-                except zmq.ZMQError as exc:
-                    if exc.errno == errno.EINTR:
-                        continue
-                    raise exc
-        except KeyboardInterrupt:
-            if self.epub_sock.closed is False:
-                self.epub_sock.setsockopt(zmq.LINGER, linger)
-                self.epub_sock.close()
-            if self.epull_sock.closed is False:
-                self.epull_sock.setsockopt(zmq.LINGER, linger)
-                self.epull_sock.close()
-            if self.context.closed is False:
-                self.context.term()
+            self.publisher.publish(package)
+            return package
+        # Add an extra fallback in case a forked process leeks through
+        except Exception:
+            log.critical('Unexpected error while polling master events',
+                         exc_info=True)
+            return None
+
+    def close(self):
+        if self._closing:
+            return
+        self._closing = True
+        if hasattr(self, 'publisher'):
+            self.publisher.close()
+        if hasattr(self, 'puller'):
+            self.puller.close()
+        if hasattr(self, 'io_loop'):
+            self.io_loop.close()
+
+    def __del__(self):
+        self.close()
 
 
-class EventReturn(multiprocessing.Process):
+class EventReturn(salt.utils.process.SignalHandlingMultiprocessingProcess):
     '''
     A dedicated process which listens to the master event bus and queues
     and forwards events to the specified returner.
     '''
-    def __init__(self, opts):
+    def __init__(self, opts, log_queue=None):
         '''
         Initialize the EventReturn system
 
         Return an EventReturn instance
         '''
-        multiprocessing.Process.__init__(self)
+        super(EventReturn, self).__init__(log_queue=log_queue)
 
         self.opts = opts
         self.event_return_queue = self.opts['event_return_queue']
@@ -964,8 +1042,23 @@ class EventReturn(multiprocessing.Process):
         self.event_queue = []
         self.stop = False
 
-    def sig_stop(self, signum, frame):
-        self.stop = True  # tell it to stop
+    # __setstate__ and __getstate__ are only used on Windows.
+    # We do this so that __init__ will be invoked on Windows in the child
+    # process so that a register_after_fork() equivalent will work on Windows.
+    def __setstate__(self, state):
+        self._is_child = True
+        self.__init__(state['opts'], log_queue=state['log_queue'])
+
+    def __getstate__(self):
+        return {'opts': self.opts,
+                'log_queue': self.log_queue}
+
+    def _handle_signals(self, signum, sigframe):
+        # Flush and terminate
+        if self.event_queue:
+            self.flush_events()
+        self.stop = True
+        super(EventReturn, self)._handle_signals(signum, sigframe)
 
     def flush_events(self):
         event_return = '{0}.event_return'.format(
@@ -975,39 +1068,36 @@ class EventReturn(multiprocessing.Process):
             try:
                 self.minion.returners[event_return](self.event_queue)
             except Exception as exc:
-                log.error('Could not store events {0}. '
-                          'Returner raised exception: {1}'.format(
-                    self.event_queue, exc))
+                log.error('Could not store events - returner \'{0}\' raised '
+                    'exception: {1}'.format(self.opts['event_return'], exc))
+                # don't waste processing power unnecessarily on converting a
+                # potentially huge dataset to a string
+                if log.level <= logging.DEBUG:
+                    log.debug('Event data that caused an exception: {0}'.format(
+                        self.event_queue))
             del self.event_queue[:]
         else:
-            log.error(
-                'Could not store return for event(s) {0}. Returner '
-                '\'{1}\' not found.'
-                    .format(self.event_queue, self.opts['event_return'])
-            )
+            log.error('Could not store return for event(s) - returner '
+                      '\'%s\' not found.', self.opts['event_return'])
 
     def run(self):
         '''
         Spin up the multiprocess event returner
         '''
-        # Properly exit if a SIGTERM is signalled
-        signal.signal(signal.SIGTERM, self.sig_stop)
-
         salt.utils.appendproctitle(self.__class__.__name__)
         self.event = get_event('master', opts=self.opts, listen=True)
         events = self.event.iter_events(full=True)
         self.event.fire_event({}, 'salt/event_listen/start')
         try:
             for event in events:
+                if event['tag'] == 'salt/event/exit':
+                    self.stop = True
                 if self._filter(event):
                     self.event_queue.append(event)
                 if len(self.event_queue) >= self.event_return_queue:
                     self.flush_events()
                 if self.stop:
                     break
-        except zmq.error.ZMQError as exc:
-            if exc.errno != errno.EINTR:  # Outside interrupt is a normal shutdown case
-                raise
         finally:  # flush all we have at this moment
             if self.event_queue:
                 self.flush_events()
